import React, {
  Component,
  PropTypes,
} from 'react';
import {
  View,
  Text,
  TouchableOpacity,
  TextInput,
} from 'react-native';

import styles from './styles.js';

class Numbers extends Component {
  constructor(props) {
    super(props);
    this.state = {
      value: this.props.value,
    };
    this.lastValue = this.props.value;
    this.check = this.check.bind(this);
  }

  componentWillReceiveProps(props) {
    this.setValue(props.value);
  }

  onPress(event, action) {
    if (this.props.readonly) return;
    const { step, max, min } = this.props;
    const { value } = this.state;
    let _value = value;
    if (action === 'plus') {
      _value += step;
      if (_value > max) return;
    } else {
      _value -= step;
      if (_value < min) return;
    }
    this.setValue(_value);
  }

  onChange(v) {
    this.setState({
      value: v,
    });
  }

  setValue(value) {
    value = Number(value);
    if (this.lastValue === value) return;
    this.lastValue = this.state.value;
    this.setState({
      value,
    });
    if (this.props.onChange) this.props.onChange(value);
  }

  check() {
    const { value } = this.state;
    const { max, min } = this.props;
    const _value = Number(value);
    // NaN
    if (max < _value || min > _value || !(_value >= 0 || _value < 0)) {
      this.setState({
        value: this.lastValue,
      });
    } else if (this.props.onChange) {
      // blur的时候同步一下
      this.props.onChange(_value);
    }
  }

  render() {
    const { style, max, min, readonly } = this.props;
    const { value } = this.state;
    const minusDisable = value <= min ? styles.disable : {};
    const plugDisable = value >= max ? styles.disable : {};

    return (
      <View style={[styles.number, style || {}]}>
        <TouchableOpacity
          style={[styles.numberButton]}
          onPress={event => this.onPress(event, 'minus')}
        >
          <Text style={[styles.numberButtonText, minusDisable]}>-</Text>
        </TouchableOpacity>
        <View style={[styles.numberValue]}>
          <TextInput
            editable={!readonly}
            value={String(value)}
            keyboardType="numeric"
            style={[styles.numberInput]}
            onBlur={this.check}
            underlineColorAndroid="transparent"
            onChangeText={v => this.onChange(v)}
          />
        </View>
        <TouchableOpacity
          style={[styles.numberButton]}
          onPress={event => this.onPress(event, 'plus')}
        >
          <Text style={[styles.numberButtonText, plugDisable]}>+</Text>
        </TouchableOpacity>
      </View>
    );
  }
}

Numbers.defaultProps = {
  max: 10000,
  min: -10000,
  step: 1,
  value: 0,
  readonly: false,
};

Numbers.propTypes = {
  max: PropTypes.number,
  min: PropTypes.number,
  step: PropTypes.number,
  value: PropTypes.number,
  readonly: PropTypes.bool,
  onChange: PropTypes.func,
};


module.exports = Numbers;
